package com.emr.service.emrCollect;

import com.emr.entity.emrCollect.Emr_Collect;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

public interface EmrCollectService {
    List<Emr_Collect> selectAllCollectTypeByCreater(Integer collectTypeFlag,HttpServletRequest request);

    List<Emr_Collect> selectAllCollectByCreater(Emr_Collect collect,String startTime,String endTime,HttpServletRequest request);

    void addCollect(Emr_Collect emrCollect, HttpServletRequest request);

    void updateCollect(Emr_Collect emrCollect,HttpServletRequest request);

    void deleteByPrimaryKey(Integer id);

    Emr_Collect selectCollectByUserIdAndPatientId(Integer userId,String patientId);
}
